@@ -18,6 +18,8 @@
 # Modified by Martin Sandve Alnes, 2009
 #
 # Test form for scalar and vector constants.
+from utils import LagrangeElement
+
 from ufl import (
     Coefficient,
     Constant,
@@ -32,13 +34,10 @@ from ufl import (
     inner,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 cell = triangle
-element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+element = LagrangeElement(cell, 1)
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 space = FunctionSpace(domain, element)
 
 v = TestFunction(space)
