@@ -2,6 +2,8 @@
 # Author: Martin Sandve Alnes
 # Date: 2008-10-03
 #
+from utils import LagrangeElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -13,12 +15,9 @@ from ufl import (
     grad,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
-element = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+element = LagrangeElement(triangle, 1, (2,))
+domain = Mesh(LagrangeElement(triangle, 1, (2,)))
 space = FunctionSpace(domain, element)
 
 u = TrialFunction(space)
