@@ -3,15 +3,14 @@
 # Author: Martin Alnes
 # Date: 2009-04-09
 #
+from utils import LagrangeElement
+
 from ufl import Coefficient, FunctionSpace, Mesh, derivative, dot, dx, grad, inner, triangle
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 cell = triangle
-X = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-Y = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+X = LagrangeElement(cell, 1, (2,))
+Y = LagrangeElement(cell, 1)
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 X_space = FunctionSpace(domain, X)
 Y_space = FunctionSpace(domain, Y)
 
