@@ -3,16 +3,15 @@
 # Author: Martin Alnes
 # Date: 2009-04-09
 #
+from utils import LagrangeElement, MixedElement
+
 from ufl import Coefficient, FunctionSpace, Mesh, derivative, dot, dx, grad, inner, split, triangle
-from ufl.finiteelement import FiniteElement, MixedElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 cell = triangle
-X = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-Y = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+X = LagrangeElement(cell, 1, (2,))
+Y = LagrangeElement(cell, 1)
 M = MixedElement([X, Y])
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 space = FunctionSpace(domain, M)
 
 u = Coefficient(space)
