@@ -2,13 +2,12 @@
 # Author: Martin Sandve Alnes
 # Date: 2008-10-28
 #
-from ufl import Coefficient, FunctionSpace, Mesh, derivative, dx, triangle
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
+from utils import LagrangeElement
 
-element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+from ufl import Coefficient, FunctionSpace, Mesh, derivative, dx, triangle
+
+element = LagrangeElement(triangle, 1)
+domain = Mesh(LagrangeElement(triangle, 1, (2,)))
 space = FunctionSpace(domain, element)
 
 u = Coefficient(space)
