@@ -16,11 +16,10 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # A mixed element of mixed elements
-from ufl import triangle
-from ufl.finiteelement import FiniteElement, MixedElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
+from utils import LagrangeElement, MixedElement
 
-P3 = FiniteElement("Lagrange", triangle, 3, (), identity_pullback, H1)
+from ufl import triangle
+
+P3 = LagrangeElement(triangle, 3)
 
 element = MixedElement([[P3, P3], [P3, P3]])
