@@ -23,6 +23,8 @@
 # a mixed formulation of Poisson's equation with BDM
 # (Brezzi-Douglas-Marini) elements.
 #
+from utils import FiniteElement, LagrangeElement, MixedElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -34,7 +36,6 @@ from ufl import (
     dx,
     triangle,
 )
-from ufl.finiteelement import FiniteElement, MixedElement
 from ufl.pullback import contravariant_piola, identity_pullback
 from ufl.sobolevspace import H1, HDiv
 
@@ -43,7 +44,7 @@ BDM1 = FiniteElement("Brezzi-Douglas-Marini", cell, 1, (2,), contravariant_piola
 DG0 = FiniteElement("Discontinuous Lagrange", cell, 0, (), identity_pullback, H1)
 
 element = MixedElement([BDM1, DG0])
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 space = FunctionSpace(domain, element)
 dg0_space = FunctionSpace(domain, DG0)
 
