@@ -3,6 +3,8 @@
 # Modified by: Martin Sandve Alnes
 # Date: 2009-02-12
 #
+from utils import FiniteElement, LagrangeElement, MixedElement
+
 from ufl import (
     FacetNormal,
     FunctionSpace,
@@ -15,7 +17,6 @@ from ufl import (
     dx,
     tetrahedron,
 )
-from ufl.finiteelement import FiniteElement, MixedElement
 from ufl.pullback import contravariant_piola, identity_pullback
 from ufl.sobolevspace import H1, HDiv
 
@@ -23,7 +24,7 @@ cell = tetrahedron
 RT = FiniteElement("Raviart-Thomas", cell, 1, (3,), contravariant_piola, HDiv)
 DG = FiniteElement("DG", cell, 0, (), identity_pullback, H1)
 MX = MixedElement([RT, DG])
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
+domain = Mesh(LagrangeElement(cell, 1, (3,)))
 space = FunctionSpace(domain, MX)
 
 (u, p) = TrialFunctions(space)
