@@ -1,3 +1,5 @@
+from utils import LagrangeElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -9,12 +11,9 @@ from ufl import (
     grad,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
-element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+element = LagrangeElement(triangle, 1)
+domain = Mesh(LagrangeElement(triangle, 1, (2,)))
 space = FunctionSpace(domain, element)
 
 v = TestFunction(space)
