@@ -21,6 +21,8 @@
 #
 # The bilinear form a(v, u) and linear form L(v) for
 # Poisson's equation in system form (vector-valued).
+from utils import LagrangeElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -33,13 +35,10 @@ from ufl import (
     inner,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 cell = triangle
-element = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+element = LagrangeElement(cell, 1, (2,))
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 space = FunctionSpace(domain, element)
 
 v = TestFunction(space)
