@@ -19,6 +19,8 @@
 # equations using a mixed formulation (Taylor-Hood elements) in
 # combination with the lhs() and rhs() operators to extract the
 # bilinear and linear forms from an expression F = 0.
+from utils import LagrangeElement, MixedElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -34,15 +36,12 @@ from ufl import (
     rhs,
     triangle,
 )
-from ufl.finiteelement import FiniteElement, MixedElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 cell = triangle
-P2 = FiniteElement("Lagrange", cell, 2, (2,), identity_pullback, H1)
-P1 = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+P2 = LagrangeElement(cell, 2, (2,))
+P1 = LagrangeElement(cell, 1)
 TH = MixedElement([P2, P1])
-domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+domain = Mesh(LagrangeElement(cell, 1, (2,)))
 space = FunctionSpace(domain, TH)
 p2_space = FunctionSpace(domain, P2)
 
