@@ -17,6 +17,8 @@
 #
 # The bilinear form a(v, u) and linear form L(v) for
 # tensor-weighted Poisson's equation.
+from utils import FiniteElement, LagrangeElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -28,13 +30,12 @@ from ufl import (
     inner,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
 from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1, L2
+from ufl.sobolevspace import L2
 
-P1 = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+P1 = LagrangeElement(triangle, 1)
 P0 = FiniteElement("Discontinuous Lagrange", triangle, 0, (2, 2), identity_pullback, L2)
-domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+domain = Mesh(LagrangeElement(triangle, 1, (2,)))
 p1_space = FunctionSpace(domain, P1)
 p0_space = FunctionSpace(domain, P0)
 
