@@ -1,10 +1,11 @@
 """Tests of the change to reference frame algorithm."""
 
+from utils import FiniteElement, LagrangeElement
+
 from ufl import Coefficient, FunctionSpace, Mesh, triangle
 from ufl.classes import Expr, ReferenceValue
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import contravariant_piola, identity_pullback
-from ufl.sobolevspace import H1, HDiv
+from ufl.pullback import contravariant_piola
+from ufl.sobolevspace import HDiv
 
 
 def change_to_reference_frame(expr):
@@ -13,11 +14,11 @@ def change_to_reference_frame(expr):
 
 
 def test_change_unmapped_form_arguments_to_reference_frame():
-    U = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    T = FiniteElement("Lagrange", triangle, 1, (2, 2), identity_pullback, H1)
+    U = LagrangeElement(triangle, 1)
+    V = LagrangeElement(triangle, 1, (2,))
+    T = LagrangeElement(triangle, 1, (2, 2))
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     u_space = FunctionSpace(domain, U)
     v_space = FunctionSpace(domain, V)
     t_space = FunctionSpace(domain, T)
@@ -33,7 +34,7 @@ def test_change_unmapped_form_arguments_to_reference_frame():
 def test_change_hdiv_form_arguments_to_reference_frame():
     V = FiniteElement("Raviart-Thomas", triangle, 1, (2,), contravariant_piola, HDiv)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     v_space = FunctionSpace(domain, V)
 
     expr = Coefficient(v_space)
@@ -43,7 +44,7 @@ def test_change_hdiv_form_arguments_to_reference_frame():
 def test_change_hcurl_form_arguments_to_reference_frame():
     V = FiniteElement("Raviart-Thomas", triangle, 1, (2,), contravariant_piola, HDiv)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     v_space = FunctionSpace(domain, V)
 
     expr = Coefficient(v_space)
