@@ -1,4 +1,5 @@
 import pytest
+from utils import LagrangeElement
 
 from ufl import (
     Coefficient,
@@ -9,7 +10,9 @@ from ufl import (
     TestFunction,
     TrialFunction,
     adjoint,
+    as_tensor,
     cofac,
+    conditional,
     conj,
     derivative,
     ds,
@@ -20,16 +23,13 @@ from ufl import (
 )
 from ufl.algorithms.check_arities import ArityMismatch
 from ufl.algorithms.compute_form_data import compute_form_data
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 
 def test_check_arities():
     # Code from bitbucket issue #49
     cell = tetrahedron
-    D = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
-    V = FunctionSpace(D, FiniteElement("Lagrange", cell, 2, (3,), identity_pullback, H1))
+    D = Mesh(LagrangeElement(cell, 1, (3,)))
+    V = FunctionSpace(D, LagrangeElement(cell, 2, (3,)))
     dv = TestFunction(V)
     du = TrialFunction(V)
 
@@ -52,8 +52,8 @@ def test_check_arities():
 
 def test_complex_arities():
     cell = tetrahedron
-    D = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
-    V = FunctionSpace(D, FiniteElement("Lagrange", cell, 2, (3,), identity_pullback, H1))
+    D = Mesh(LagrangeElement(cell, 1, (3,)))
+    V = FunctionSpace(D, LagrangeElement(cell, 2, (3,)))
     v = TestFunction(V)
     u = TrialFunction(V)
 
@@ -72,8 +72,8 @@ def test_complex_arities():
 
 def test_product_arity():
     cell = tetrahedron
-    D = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
-    V = FunctionSpace(D, FiniteElement("Lagrange", cell, 2, (3,), identity_pullback, H1))
+    D = Mesh(LagrangeElement(cell, 1, (3,)))
+    V = FunctionSpace(D, LagrangeElement(cell, 2, (3,)))
     v = TestFunction(V)
     u = TrialFunction(V)
 
@@ -84,3 +84,41 @@ def test_product_arity():
     with pytest.raises(ArityMismatch):
         L = inner(v, v) * dx
         compute_form_data(L, complex_mode=False)
+
+
+def test_zero_simplify_arity():
+    """
+    Test that adding verious zero-like expressions to a form is simplified,
+    such that one can compute form data for the integral.
+    """
+    cell = tetrahedron
+    D = Mesh(LagrangeElement(cell, 1, (3,)))
+    V = FunctionSpace(D, LagrangeElement(cell, 2))
+    v = TestFunction(V)
+    u = Coefficient(V)
+
+    nonzero = 1
+    with pytest.raises(ArityMismatch):
+        F = inner(u, v + nonzero) * dx
+        compute_form_data(F)
+    z = Coefficient(V)
+
+    # Add a Zero-component (rank-0) of a tensor to a rank-1 tensor
+    zero = as_tensor([0, z])[0]
+    F = inner(u, v + zero) * dx
+    fd = compute_form_data(F)
+    assert fd.num_coefficients == 1
+
+    # Add a conditional that should have been simplified to zero (rank-0)
+    # to a rank-1 tensor
+    zero = conditional(z < 0, 0, 0)
+    F = inner(u, v + zero) * dx
+    fd = compute_form_data(F)
+    assert fd.num_coefficients == 1
+
+    # Check that nested zero conditionals are simplifed to zero (rank-0)
+    # and can be added to a rank-1 tensor
+    zero = conditional(z < 0, 0, conditional(z == 0, 0, 0))
+    F = inner(u, v + zero) * dx
+    fd = compute_form_data(F)
+    assert fd.num_coefficients == 1
