@@ -1,6 +1,7 @@
 import cmath
 
 import pytest
+from utils import LagrangeElement
 
 from ufl import (
     Coefficient,
@@ -41,9 +42,6 @@ from ufl.algorithms.comparison_checker import ComplexComparisonError, do_compari
 from ufl.algorithms.formtransformations import compute_form_adjoint
 from ufl.algorithms.remove_complex_nodes import remove_complex_nodes
 from ufl.constantvalue import ComplexValue, Zero
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 
 def test_conj(self):
@@ -77,8 +75,8 @@ def test_imag(self):
 
 def test_compute_form_adjoint(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     u = TrialFunction(space)
@@ -111,8 +109,8 @@ def test_complex_algebra(self):
 
 def test_automatic_simplification(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -125,8 +123,8 @@ def test_automatic_simplification(self):
 
 def test_apply_algebra_lowering_complex(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -156,8 +154,8 @@ def test_apply_algebra_lowering_complex(self):
 
 def test_remove_complex_nodes(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     u = TrialFunction(space)
@@ -179,8 +177,8 @@ def test_remove_complex_nodes(self):
 
 def test_comparison_checker(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     u = TrialFunction(space)
@@ -207,8 +205,8 @@ def test_comparison_checker(self):
 
 def test_complex_degree_handling(self):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 3, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 3)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
