@@ -3,6 +3,8 @@ __date__ = "2009-02-17 -- 2009-02-17"
 
 from itertools import chain
 
+from utils import FiniteElement, LagrangeElement, MixedElement
+
 from ufl import (
     CellDiameter,
     CellVolume,
@@ -16,6 +18,7 @@ from ufl import (
     Index,
     Jacobian,
     JacobianInverse,
+    Label,
     Mesh,
     SpatialCoordinate,
     TestFunction,
@@ -60,9 +63,9 @@ from ufl.algorithms.apply_algebra_lowering import apply_algebra_lowering
 from ufl.algorithms.apply_derivatives import apply_derivatives
 from ufl.algorithms.apply_geometry_lowering import apply_geometry_lowering
 from ufl.classes import Indexed, MultiIndex, ReferenceGrad
-from ufl.constantvalue import as_ufl
+from ufl.constantvalue import Zero, as_ufl
 from ufl.domain import extract_unique_domain
-from ufl.finiteelement import FiniteElement, MixedElement
+from ufl.operators import Variable
 from ufl.pullback import identity_pullback
 from ufl.sobolevspace import H1, L2
 
@@ -89,7 +92,7 @@ def assertEqualBySampling(actual, expected):
         if c.ufl_shape == ():
             return z * (0.1 + 0.9 * m / n)
         elif len(c.ufl_shape) == 1:
-            return tuple((z * (j + 0.1 + 0.9 * m / n) for j in range(c.ufl_shape[0])))
+            return tuple(z * (j + 0.1 + 0.9 * m / n) for j in range(c.ufl_shape[0]))
         else:
             raise NotImplementedError("Tensor valued expressions not supported here.")
 
@@ -118,10 +121,10 @@ def assertEqualBySampling(actual, expected):
     if not av == bv:
         print("Tried to sample expressions to compare but failed:")
         print()
-        print((str(a)))
+        print(str(a))
         print(av)
         print()
-        print((str(b)))
+        print(str(b))
         print(bv)
         print()
 
@@ -130,8 +133,8 @@ def assertEqualBySampling(actual, expected):
 
 def _test(self, f, df):
     cell = triangle
-    element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -198,8 +201,8 @@ def testArgument(self):
     def f(w):
         return TestFunction(
             FunctionSpace(
-                Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)),
-                FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1),
+                Mesh(LagrangeElement(triangle, 1, (2,))),
+                LagrangeElement(triangle, 1),
             )
         )
 
@@ -214,9 +217,7 @@ def testArgument(self):
 
 def testSpatialCoordinate(self):
     def f(w):
-        return SpatialCoordinate(
-            Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
-        )[0]
+        return SpatialCoordinate(Mesh(LagrangeElement(triangle, 1, (2,))))[0]
 
     def df(w, v):
         return zero()
@@ -226,9 +227,7 @@ def testSpatialCoordinate(self):
 
 def testFacetNormal(self):
     def f(w):
-        return FacetNormal(
-            Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
-        )[0]
+        return FacetNormal(Mesh(LagrangeElement(triangle, 1, (2,))))[0]
 
     def df(w, v):
         return zero()
@@ -238,7 +237,7 @@ def testFacetNormal(self):
 
 def testFacetArea(self):
     def f(w):
-        return FacetArea(Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)))
+        return FacetArea(Mesh(LagrangeElement(triangle, 1, (2,))))
 
     def df(w, v):
         return zero()
@@ -248,9 +247,7 @@ def testFacetArea(self):
 
 def testCellDiameter(self):
     def f(w):
-        return CellDiameter(
-            Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
-        )
+        return CellDiameter(Mesh(LagrangeElement(triangle, 1, (2,))))
 
     def df(w, v):
         return zero()
@@ -260,9 +257,7 @@ def testCellDiameter(self):
 
 def testCircumradius(self):
     def f(w):
-        return Circumradius(
-            Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
-        )
+        return Circumradius(Mesh(LagrangeElement(triangle, 1, (2,))))
 
     def df(w, v):
         return zero()
@@ -272,7 +267,7 @@ def testCircumradius(self):
 
 def testCellVolume(self):
     def f(w):
-        return CellVolume(Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)))
+        return CellVolume(Mesh(LagrangeElement(triangle, 1, (2,))))
 
     def df(w, v):
         return zero()
@@ -489,8 +484,8 @@ def testListTensor(self):
 
 def test_single_scalar_coefficient_derivative(self):
     cell = triangle
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, V)
     u = Coefficient(space)
     v = TestFunction(space)
@@ -501,8 +496,8 @@ def test_single_scalar_coefficient_derivative(self):
 
 def test_single_vector_coefficient_derivative(self):
     cell = triangle
-    V = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(cell, 1, (2,))
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, V)
     u = Coefficient(space)
     v = TestFunction(space)
@@ -514,10 +509,10 @@ def test_single_vector_coefficient_derivative(self):
 
 def test_multiple_coefficient_derivative(self):
     cell = triangle
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    W = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
+    V = LagrangeElement(cell, 1)
+    W = LagrangeElement(cell, 1, (2,))
     M = MixedElement([V, W])
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     v_space = FunctionSpace(domain, V)
     w_space = FunctionSpace(domain, W)
     m_space = FunctionSpace(domain, M)
@@ -540,9 +535,9 @@ def test_multiple_coefficient_derivative(self):
 def test_indexed_coefficient_derivative(self):
     cell = triangle
     ident = Identity(2)
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    W = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(cell, 1)
+    W = LagrangeElement(cell, 1, (2,))
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     v_space = FunctionSpace(domain, V)
     w_space = FunctionSpace(domain, W)
     u = Coefficient(w_space)
@@ -562,10 +557,10 @@ def test_indexed_coefficient_derivative(self):
 
 def test_multiple_indexed_coefficient_derivative(self):
     cell = tetrahedron
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+    V = LagrangeElement(cell, 1)
     V2 = MixedElement([V, V])
-    W = FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
+    W = LagrangeElement(cell, 1, (3,))
+    domain = Mesh(LagrangeElement(cell, 1, (3,)))
     v2_space = FunctionSpace(domain, V2)
     w_space = FunctionSpace(domain, W)
     u = Coefficient(w_space)
@@ -581,10 +576,10 @@ def test_multiple_indexed_coefficient_derivative(self):
 
 def test_segregated_derivative_of_convection(self):
     cell = tetrahedron
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    W = FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1)
+    V = LagrangeElement(cell, 1)
+    W = LagrangeElement(cell, 1, (3,))
 
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (3,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (3,)))
     v_space = FunctionSpace(domain, V)
     w_space = FunctionSpace(domain, W)
 
@@ -621,9 +616,9 @@ def test_segregated_derivative_of_convection(self):
 
 
 def test_coefficient_derivatives(self):
-    V = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+    V = LagrangeElement(triangle, 1)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, V)
 
     dv = TestFunction(space)
@@ -647,10 +642,10 @@ def test_coefficient_derivatives(self):
 
 
 def test_vector_coefficient_scalar_derivatives(self):
-    V = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+    V = LagrangeElement(triangle, 1)
     VV = FiniteElement("vector Lagrange", triangle, 1, (2,), identity_pullback, H1)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     v_space = FunctionSpace(domain, V)
     vv_space = FunctionSpace(domain, VV)
 
@@ -664,7 +659,7 @@ def test_vector_coefficient_scalar_derivatives(self):
 
     integrand = inner(f, g)
 
-    i0, i1, i2, i3, i4 = [Index(count=c) for c in range(5)]
+    i0, i1, _i2, _i3, _i4 = [Index(count=c) for c in range(5)]
     expected = as_tensor(df[i1] * dv, (i1,))[i0] * g[i0]
 
     F = integrand * dx
@@ -675,10 +670,10 @@ def test_vector_coefficient_scalar_derivatives(self):
 
 
 def test_vector_coefficient_derivatives(self):
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    VV = FiniteElement("Lagrange", triangle, 1, (2, 2), identity_pullback, H1)
+    V = LagrangeElement(triangle, 1, (2,))
+    VV = LagrangeElement(triangle, 1, (2, 2))
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     v_space = FunctionSpace(domain, V)
     vv_space = FunctionSpace(domain, VV)
 
@@ -692,7 +687,7 @@ def test_vector_coefficient_derivatives(self):
 
     integrand = inner(f, g)
 
-    i0, i1, i2, i3, i4 = [Index(count=c) for c in range(5)]
+    i0, i1, i2, _i3, _i4 = [Index(count=c) for c in range(5)]
     expected = as_tensor(df[i2, i1] * dv[i1], (i2,))[i0] * g[i0]
 
     F = integrand * dx
@@ -704,10 +699,10 @@ def test_vector_coefficient_derivatives(self):
 
 
 def test_vector_coefficient_derivatives_of_product(self):
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    VV = FiniteElement("Lagrange", triangle, 1, (2, 2), identity_pullback, H1)
+    V = LagrangeElement(triangle, 1, (2,))
+    VV = LagrangeElement(triangle, 1, (2, 2))
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     v_space = FunctionSpace(domain, V)
     vv_space = FunctionSpace(domain, VV)
 
@@ -748,8 +743,8 @@ def test_vector_coefficient_derivatives_of_product(self):
 
 def testHyperElasticity(self):
     cell = interval
-    element = FiniteElement("Lagrange", cell, 2, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (1,), identity_pullback, H1))
+    element = LagrangeElement(cell, 2)
+    domain = Mesh(LagrangeElement(cell, 1, (1,)))
     space = FunctionSpace(domain, element)
     w = Coefficient(space)
     v = TestFunction(space)
@@ -826,9 +821,9 @@ def testHyperElasticity(self):
 
 def test_mass_derived_from_functional(self):
     cell = triangle
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+    V = LagrangeElement(cell, 1)
 
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, V)
 
     v = TestFunction(space)
@@ -849,9 +844,9 @@ def test_mass_derived_from_functional(self):
 
 def test_derivative_replace_works_together(self):
     cell = triangle
-    V = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+    V = LagrangeElement(cell, 1)
 
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (2,)))
     space = FunctionSpace(domain, V)
 
     v = TestFunction(space)
@@ -874,7 +869,7 @@ def test_derivative_replace_works_together(self):
 
 
 def test_index_simplification_handles_repeated_indices(self):
-    mesh = Mesh(FiniteElement("Lagrange", quadrilateral, 1, (2,), identity_pullback, H1))
+    mesh = Mesh(LagrangeElement(quadrilateral, 1, (2,)))
     V = FunctionSpace(mesh, FiniteElement("DQ", quadrilateral, 0, (2, 2), identity_pullback, L2))
     K = JacobianInverse(mesh)
     G = outer(Identity(2), Identity(2))
@@ -893,7 +888,7 @@ def test_index_simplification_handles_repeated_indices(self):
 
 
 def test_index_simplification_reference_grad(self):
-    mesh = Mesh(FiniteElement("Lagrange", quadrilateral, 1, (2,), identity_pullback, H1))
+    mesh = Mesh(LagrangeElement(quadrilateral, 1, (2,)))
     (i,) = indices(1)
     A = as_tensor(Indexed(Jacobian(mesh), MultiIndex((i, i))), (i,))
     expr = apply_derivatives(apply_geometry_lowering(apply_algebra_lowering(A[0])))
@@ -902,12 +897,45 @@ def test_index_simplification_reference_grad(self):
     assert expr.ufl_shape == ()
 
 
+def test_zero_shape(self):
+    cell = triangle
+    shape = (2, 3, 4)
+    P1 = FiniteElement("Lagrange", cell, 1, shape, identity_pullback, H1)
+    domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+    V = FunctionSpace(domain, P1)
+    v = TestFunction(V)
+    u = Coefficient(V)
+    w = Coefficient(V)
+
+    (i,) = indices(1)
+    z = zero(shape)
+    zi = z[:, i, :]
+    wi = w[:, i, :]
+    assert isinstance(zi, Zero)
+    assert wi.ufl_shape == (shape[0], shape[-1])
+    assert wi.ufl_shape == zi.ufl_shape
+
+    a = derivative(conditional(u[0, 0, 0] < 1, zi, wi), u, v)
+    assert not isinstance(a, Zero)
+
+    assert a.ufl_shape == zi.ufl_shape
+    assert a.ufl_free_indices == zi.ufl_free_indices
+    assert a.ufl_index_dimensions == zi.ufl_index_dimensions
+
+    expr = apply_derivatives(apply_geometry_lowering(apply_algebra_lowering(a)))
+
+    assert isinstance(expr, Zero)
+    assert expr.ufl_shape == a.ufl_shape
+    assert expr.ufl_free_indices == a.ufl_free_indices
+    assert expr.ufl_index_dimensions == a.ufl_index_dimensions
+
+
 # --- Scratch space
 
 
 def test_foobar(self):
-    element = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
     v = TestFunction(space)
     du = TrialFunction(space)
@@ -925,3 +953,34 @@ def test_foobar(self):
     L = NS_a(U, v) * dx
     _ = derivative(L, U, du)
     # TODO: assert something
+
+
+def test_variable_label():
+    """Check that derivatives respect variable labels"""
+    element = LagrangeElement(triangle, 1, ())
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
+    space = FunctionSpace(domain, element)
+    u = TrialFunction(space)
+    F = sin(u)
+    F_var = variable(F)
+    C = F_var**2
+
+    # Check that standard variable gives correct result
+    ref_expr = 2.0 * F_var
+    dCdF = apply_derivatives(diff(C, F_var))
+    assert dCdF == ref_expr
+
+    # Check that variable with same label gives same result
+    F_var_2 = Variable(F, label=F_var.label())
+    dCdF_2 = apply_derivatives(diff(C, F_var_2))
+    assert dCdF_2 == ref_expr
+
+    # Check that new variable gives 0 result
+    F_var_3 = variable(F)
+    dCdF_3 = apply_derivatives(diff(C, F_var_3))
+    assert dCdF_3 == 0
+
+    # Check that variable with different label gives 0 result
+    F_var_4 = Variable(F, label=Label(888))
+    dCdF_4 = apply_derivatives(diff(C, F_var_4))
+    assert dCdF_4 == 0
