@@ -7,6 +7,7 @@ __date__ = "2009-03-19 -- 2012-03-20"
 import math
 
 import pytest
+from utils import LagrangeElement
 
 from ufl import (
     Coefficient,
@@ -35,9 +36,6 @@ from ufl import (
 )
 from ufl.algorithms import compute_form_data, expand_derivatives, expand_indices
 from ufl.algorithms.renumbering import renumber_indices
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 # TODO: Test expand_indices2 throuroughly for correctness, then efficiency:
 # expand_indices, expand_indices2 = expand_indices2, expand_indices
@@ -46,10 +44,10 @@ from ufl.sobolevspace import H1
 class Fixture:
     def __init__(self):
         cell = triangle
-        element = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-        velement = FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1)
-        telement = FiniteElement("Lagrange", cell, 1, (2, 2), identity_pullback, H1)
-        domain = Mesh(FiniteElement("Lagrange", cell, 1, (2,), identity_pullback, H1))
+        element = LagrangeElement(cell, 1)
+        velement = LagrangeElement(cell, 1, (2,))
+        telement = LagrangeElement(cell, 1, (2, 2))
+        domain = Mesh(LagrangeElement(cell, 1, (2,)))
         space = FunctionSpace(domain, element)
         vspace = FunctionSpace(domain, velement)
         tspace = FunctionSpace(domain, telement)
