@@ -13,6 +13,8 @@ __license__ = "GNU GPL version 3 or any later version"
 # Examples copied from the FFC demo directory, examples contributed
 # by Johan Jansson, Kristian Oelgaard, Marie Rognes, and Garth Wells.
 
+from utils import FiniteElement, LagrangeElement, MixedElement
+
 from ufl import (
     Coefficient,
     Constant,
@@ -43,14 +45,13 @@ from ufl import (
     tetrahedron,
     triangle,
 )
-from ufl.finiteelement import FiniteElement, MixedElement
 from ufl.pullback import contravariant_piola, covariant_piola, identity_pullback
-from ufl.sobolevspace import H1, L2, HCurl, HDiv
+from ufl.sobolevspace import L2, HCurl, HDiv
 
 
 def testConstant():
-    element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -66,8 +67,8 @@ def testConstant():
 
 
 def testElasticity():
-    element = FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 1, (3,))
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -82,8 +83,8 @@ def testElasticity():
 
 
 def testEnergyNorm():
-    element = FiniteElement("Lagrange", tetrahedron, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 1)
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
 
     v = Coefficient(space)
@@ -91,8 +92,8 @@ def testEnergyNorm():
 
 
 def testEquation():
-    element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     k = 0.1
@@ -108,8 +109,8 @@ def testEquation():
 
 
 def testFunctionOperators():
-    element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -123,8 +124,8 @@ def testFunctionOperators():
 
 
 def testHeat():
-    element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -139,8 +140,8 @@ def testHeat():
 
 
 def testMass():
-    element = FiniteElement("Lagrange", tetrahedron, 3, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 3)
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
     v = TestFunction(space)
     u = TrialFunction(space)
@@ -148,7 +149,7 @@ def testMass():
 
 
 def testMixedMixedElement():
-    P3 = FiniteElement("Lagrange", triangle, 3, (), identity_pullback, H1)
+    P3 = LagrangeElement(triangle, 3)
     MixedElement([[P3, P3], [P3, P3]])
 
 
@@ -158,7 +159,7 @@ def testMixedPoisson():
     DG = FiniteElement("Discontinuous Lagrange", triangle, q - 1, (), identity_pullback, L2)
 
     mixed_element = MixedElement([BDM, DG])
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, mixed_element)
 
     (tau, w) = TestFunctions(space)
@@ -169,8 +170,8 @@ def testMixedPoisson():
 
 
 def testNavierStokes():
-    element = FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 1, (3,))
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -182,8 +183,8 @@ def testNavierStokes():
 
 
 def testNeumannProblem():
-    element = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -199,8 +200,8 @@ def testNeumannProblem():
 
 
 def testOptimization():
-    element = FiniteElement("Lagrange", triangle, 3, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 3)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
     v = TestFunction(space)
     u = TrialFunction(space)
@@ -210,16 +211,16 @@ def testOptimization():
 
 
 def testP5tet():
-    FiniteElement("Lagrange", tetrahedron, 5, (), identity_pullback, H1)
+    LagrangeElement(tetrahedron, 5)
 
 
 def testP5tri():
-    FiniteElement("Lagrange", triangle, 5, (), identity_pullback, H1)
+    LagrangeElement(triangle, 5)
 
 
 def testPoissonDG():
     element = FiniteElement("Discontinuous Lagrange", triangle, 1, (), identity_pullback, L2)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -255,8 +256,8 @@ def testPoissonDG():
 
 
 def testPoisson():
-    element = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -269,8 +270,8 @@ def testPoisson():
 
 
 def testPoissonSystem():
-    element = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    element = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -289,8 +290,8 @@ def testProjection():
     # in FFC for a while. For DOLFIN, the current (global) L^2
     # projection can be extended to handle also local projections.
 
-    P1 = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    P1 = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, P1)
     _ = TestFunction(space)
     _ = Coefficient(space)
@@ -303,7 +304,7 @@ def testProjection():
 
 
 def testQuadratureElement():
-    element = FiniteElement("Lagrange", triangle, 2, (), identity_pullback, H1)
+    element = LagrangeElement(triangle, 2)
 
     # FFC notation:
     # QE = QuadratureElement(triangle, 3)
@@ -312,7 +313,7 @@ def testQuadratureElement():
     QE = FiniteElement("Quadrature", triangle, 3, (), identity_pullback, L2)
     sig = FiniteElement("Quadrature", triangle, 3, (2,), identity_pullback, L2)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -329,11 +330,11 @@ def testQuadratureElement():
 def testStokes():
     # UFLException: Shape mismatch in sum.
 
-    P2 = FiniteElement("Lagrange", triangle, 2, (2,), identity_pullback, H1)
-    P1 = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+    P2 = LagrangeElement(triangle, 2, (2,))
+    P1 = LagrangeElement(triangle, 1)
     TH = MixedElement([P2, P1])
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     th_space = FunctionSpace(domain, TH)
     p2_space = FunctionSpace(domain, P2)
 
@@ -348,16 +349,16 @@ def testStokes():
 
 
 def testSubDomain():
-    element = FiniteElement("Lagrange", tetrahedron, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 1)
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
     f = Coefficient(space)
     _ = f * dx(2) + f * ds(5)
 
 
 def testSubDomains():
-    element = FiniteElement("Lagrange", tetrahedron, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    element = LagrangeElement(tetrahedron, 1)
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     space = FunctionSpace(domain, element)
 
     v = TestFunction(space)
@@ -368,7 +369,7 @@ def testSubDomains():
 
 def testTensorWeightedPoisson():
     # FFC notation:
-    # P1 = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+    # P1 = LagrangeElement(triangle, 1)
     # P0 = FiniteElement("Discontinuous Lagrange", triangle, 0, (), identity_pullback, L2)
     #
     # v = TestFunction(P1)
@@ -384,10 +385,10 @@ def testTensorWeightedPoisson():
     #
     # a = dot(grad(v), mult(C, grad(u)))*dx
 
-    P1 = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
+    P1 = LagrangeElement(triangle, 1)
     P0 = FiniteElement("Discontinuous Lagrange", triangle, 0, (2, 2), identity_pullback, L2)
 
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     p1_space = FunctionSpace(domain, P1)
     p0_space = FunctionSpace(domain, P0)
 
@@ -420,14 +421,14 @@ def testVectorLaplaceGradCurl():
     shape = tetrahedron
     order = 1
 
-    GRAD = FiniteElement("Lagrange", shape, order, (), identity_pullback, H1)
+    GRAD = LagrangeElement(shape, order, ())
 
     # FFC notation: CURL = FiniteElement("Nedelec", shape, order-1)
     CURL = FiniteElement("N1curl", shape, order, (3,), covariant_piola, HCurl)
 
-    VectorLagrange = FiniteElement("Lagrange", shape, order + 1, (3,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", shape, 1, (3,), identity_pullback, H1))
+    VectorLagrange = LagrangeElement(shape, order + 1, (3,))
+    domain = Mesh(LagrangeElement(shape, 1, (3,)))
 
-    [a, L] = HodgeLaplaceGradCurl(
+    [_a, _L] = HodgeLaplaceGradCurl(
         FunctionSpace(domain, MixedElement([GRAD, CURL])), FunctionSpace(domain, VectorLagrange)
     )
