@@ -4,12 +4,14 @@ __authors__ = "Nacime Bouziani"
 __date__ = "2021-11-19"
 
 import pytest
+from utils import FiniteElement, LagrangeElement
 
 from ufl import (
     Action,
     Adjoint,
     Argument,
     Coefficient,
+    Cofunction,
     FunctionSpace,
     Mesh,
     TestFunction,
@@ -26,20 +28,19 @@ from ufl import (
 from ufl.algorithms.ad import expand_derivatives
 from ufl.algorithms.analysis import (
     extract_arguments,
-    extract_arguments_and_coefficients,
     extract_base_form_operators,
     extract_coefficients,
+    extract_terminals_with_domain,
 )
 from ufl.algorithms.expand_indices import expand_indices
 from ufl.core.interpolate import Interpolate
-from ufl.finiteelement import FiniteElement
 from ufl.pullback import identity_pullback
 from ufl.sobolevspace import H1
 
 
 @pytest.fixture
 def domain_2d():
-    return Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    return Mesh(LagrangeElement(triangle, 1, (2,)))
 
 
 @pytest.fixture
@@ -55,11 +56,8 @@ def V2(domain_2d):
 
 
 def test_symbolic(V1, V2):
-    # Set dual of V2
-    V2_dual = V2.dual()
-
     u = Coefficient(V1)
-    vstar = Argument(V2_dual, 0)
+    vstar = Argument(V2.dual(), 0)
     Iu = Interpolate(u, vstar)
 
     assert Iu == Interpolate(u, V2)
@@ -69,6 +67,20 @@ def test_symbolic(V1, V2):
     assert Iu.ufl_operands == (u,)
 
 
+def test_symbolic_adjoint(V1, V2):
+    u = Argument(V1, 0)
+    form = inner(1, Argument(V2, 0)) * dx
+    cofun = Cofunction(V2.dual())
+
+    for vstar in (form, cofun):
+        Iu = Interpolate(u, vstar)
+
+        assert Iu.ufl_function_space() == V1.dual()
+        assert Iu.argument_slots() == (vstar, u)
+        assert Iu.arguments() == (u,)
+        assert Iu.ufl_operands == (u,)
+
+
 def test_action_adjoint(V1, V2):
     # Set dual of V2
     V2_dual = V2.dual()
@@ -157,12 +169,12 @@ def test_extract_base_form_operators(V1, V2):
     # -- Interpolate(u, V2) -- #
     Iu = Interpolate(u, V2)
     assert extract_arguments(Iu) == [vstar]
-    assert extract_arguments_and_coefficients(Iu) == ([vstar], [u])
+    assert extract_terminals_with_domain(Iu) == ([vstar], [u], [])
 
     F = Iu * dx
     # Form composition: Iu * dx <=> Action(v * dx, Iu(u; v*))
     assert extract_arguments(F) == []
-    assert extract_arguments_and_coefficients(F) == ([], [u])
+    assert extract_terminals_with_domain(F) == ([], [u], [])
 
     for e in [Iu, F]:
         assert extract_coefficients(e) == [u]
@@ -171,7 +183,7 @@ def test_extract_base_form_operators(V1, V2):
     # -- Interpolate(u, V2) -- #
     Iv = Interpolate(uhat, V2)
     assert extract_arguments(Iv) == [vstar, uhat]
-    assert extract_arguments_and_coefficients(Iv) == ([vstar, uhat], [])
+    assert extract_terminals_with_domain(Iv) == ([vstar, uhat], [], [])
     assert extract_coefficients(Iv) == []
     assert extract_base_form_operators(Iv) == [Iv]
 
@@ -180,3 +192,55 @@ def test_extract_base_form_operators(V1, V2):
     v = TestFunction(V1)
     F = Action(v * v2 * dx, Iv)
     assert extract_arguments(F) == [v, uhat]
+
+
+def test_operator_derivative_reconstruction(V1, V2):
+    u = Coefficient(V1)
+
+    # Define Interpolate
+    Iu = Interpolate(u, V2)
+
+    # -- Differentiate: Interpolate(u, V2) -- #
+    uhat = TrialFunction(V1)
+    dIu = derivative(Iu, u, uhat)
+    # Check operator derivative can be reconstructed
+    dIu_r = dIu._ufl_expr_reconstruct_(*dIu.ufl_operands)
+    dIu = expand_derivatives(dIu_r)
+
+    assert dIu == Interpolate(uhat, V2)
+
+
+def test_interpolate_argument_numbering(V1, V2):
+    u = Coefficient(V1)
+    u0 = Argument(V1, 0)
+    u1 = Argument(V1, 1)
+    vstar0 = Argument(V2.dual(), 0)
+    vstar1 = Argument(V2.dual(), 1)
+    cofunc = Cofunction(V2.dual())
+    one_form = Argument(V2, 0) * dx
+
+    Interpolate(u, cofunc)
+    Interpolate(u, vstar0)
+    Interpolate(u, V2)
+    Interpolate(u0, cofunc)  # adjoint
+    Interpolate(u0, one_form)  # adjoint
+    Interpolate(u1, vstar0)
+    Interpolate(u0, vstar1)  # adjoint
+
+    with pytest.raises(ValueError, match=r"Same argument numbers in first and second operands"):
+        Interpolate(u0, vstar0)
+
+    with pytest.raises(ValueError, match=r"Non-contiguous argument numbers in interpolate."):
+        Interpolate(u, vstar1)
+
+    with pytest.raises(ValueError, match=r"Non-contiguous argument numbers in interpolate."):
+        Interpolate(u1, cofunc)
+
+    with pytest.raises(ValueError, match=r"Non-contiguous argument numbers in interpolate."):
+        Interpolate(u1, one_form)
+
+    u2 = u0 * u1
+    with pytest.raises(
+        ValueError, match=r"Can only interpolate expressions with zero or one argument."
+    ):
+        Interpolate(u2, vstar0)
