@@ -3,6 +3,8 @@ __authors__ = "Marie E. Rognes"
 # First added: 2011-11-09
 # Last changed: 2011-11-09
 
+from utils import LagrangeElement
+
 from ufl import (
     Argument,
     Coefficient,
@@ -20,14 +22,11 @@ from ufl import (
     interval,
     system,
 )
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 
 
 def test_lhs_rhs_simple():
-    V = FiniteElement("Lagrange", interval, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", interval, 1, (1,), identity_pullback, H1))
+    V = LagrangeElement(interval, 1)
+    domain = Mesh(LagrangeElement(interval, 1, (1,)))
     space = FunctionSpace(domain, V)
     v = TestFunction(space)
     u = TrialFunction(space)
@@ -55,8 +54,8 @@ def test_lhs_rhs_simple():
 
 
 def test_lhs_rhs_derivatives():
-    V = FiniteElement("Lagrange", interval, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", interval, 1, (1,), identity_pullback, H1))
+    V = LagrangeElement(interval, 1)
+    domain = Mesh(LagrangeElement(interval, 1, (1,)))
     space = FunctionSpace(domain, V)
     v = TestFunction(space)
     u = TrialFunction(space)
@@ -72,8 +71,8 @@ def test_lhs_rhs_derivatives():
 
 
 def test_lhs_rhs_slightly_obscure():
-    V = FiniteElement("Lagrange", interval, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", interval, 1, (1,), identity_pullback, H1))
+    V = LagrangeElement(interval, 1)
+    domain = Mesh(LagrangeElement(interval, 1, (1,)))
     space = FunctionSpace(domain, V)
     u = TrialFunction(space)
     w = Argument(space, 2)
