@@ -1,3 +1,5 @@
+from utils import FiniteElement, LagrangeElement
+
 from ufl import (
     CellVolume,
     Coefficient,
@@ -28,9 +30,8 @@ from ufl.algorithms.apply_derivatives import (
     apply_derivatives,
 )
 from ufl.algorithms.renumbering import renumber_indices
-from ufl.finiteelement import FiniteElement
 from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1, L2
+from ufl.sobolevspace import L2
 
 # Note: the old tests in test_automatic_differentiation.py are a bit messy
 #       but still cover many things that are not in here yet.
@@ -44,9 +45,9 @@ def test_apply_derivatives_doesnt_change_expression_without_derivatives():
     cell = triangle
     d = 2
     V0 = FiniteElement("Discontinuous Lagrange", cell, 0, (), identity_pullback, L2)
-    V1 = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+    V1 = LagrangeElement(cell, 1)
 
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (d,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (d,)))
     v0_space = FunctionSpace(domain, V0)
     v1_space = FunctionSpace(domain, V1)
 
@@ -113,8 +114,8 @@ def test_literal_derivatives_are_zero():
             assert apply_derivatives(diff(lit, v)) == zero(lit.ufl_shape + v.ufl_shape)
 
     V0 = FiniteElement("Discontinuous Lagrange", cell, 0, (), identity_pullback, L2)
-    V1 = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (d,), identity_pullback, H1))
+    V1 = LagrangeElement(cell, 1)
+    domain = Mesh(LagrangeElement(cell, 1, (d,)))
     v0_space = FunctionSpace(domain, V0)
     v1_space = FunctionSpace(domain, V1)
     u0 = Coefficient(v0_space)
@@ -138,13 +139,13 @@ def test_grad_ruleset():
     d = 2
 
     V0 = FiniteElement("Discontinuous Lagrange", cell, 0, (), identity_pullback, L2)
-    V1 = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
-    V2 = FiniteElement("Lagrange", cell, 2, (), identity_pullback, H1)
+    V1 = LagrangeElement(cell, 1)
+    V2 = LagrangeElement(cell, 2)
     W0 = FiniteElement("Discontinuous Lagrange", cell, 0, (2,), identity_pullback, L2)
-    W1 = FiniteElement("Lagrange", cell, 1, (d,), identity_pullback, H1)
-    W2 = FiniteElement("Lagrange", cell, 2, (d,), identity_pullback, H1)
+    W1 = LagrangeElement(cell, 1, (d,))
+    W2 = LagrangeElement(cell, 2, (d,))
 
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (d,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(cell, 1, (d,)))
     v0_space = FunctionSpace(domain, V0)
     v1_space = FunctionSpace(domain, V1)
     v2_space = FunctionSpace(domain, V2)
