@@ -1,6 +1,7 @@
 """Test the is_cellwise_constant function on all relevant terminal types."""
 
 import pytest
+from utils import FiniteElement, LagrangeElement
 
 from ufl import (
     Cell,
@@ -32,8 +33,10 @@ from ufl.classes import (
     FacetJacobian,
     FacetJacobianDeterminant,
     FacetJacobianInverse,
+    RidgeJacobian,
+    RidgeJacobianDeterminant,
+    RidgeJacobianInverse,
 )
-from ufl.finiteelement import FiniteElement
 from ufl.pullback import identity_pullback
 from ufl.sobolevspace import H1, L2, HInf
 
@@ -262,7 +265,7 @@ def test_coordinates_never_cellwise_constant(domains):
 
 def test_coordinates_never_cellwise_constant_vertex():
     # The only exception here:
-    domains = Mesh(FiniteElement("Lagrange", Cell("vertex"), 1, (3,), identity_pullback, H1))
+    domains = Mesh(LagrangeElement(Cell("vertex"), 1, (3,)))
     assert domains.ufl_cell().cellname() == "vertex"
     e = SpatialCoordinate(domains)
     assert is_cellwise_constant(e)
@@ -284,6 +287,13 @@ def mappings_are_cellwise_constant(domain, test):
         assert is_cellwise_constant(e) == test
         e = FacetJacobianInverse(domain)
         assert is_cellwise_constant(e) == test
+    if domain.topological_dimension() > 2:
+        e = RidgeJacobian(domain)
+        assert is_cellwise_constant(e) == test
+        e = RidgeJacobianDeterminant(domain)
+        assert is_cellwise_constant(e) == test
+        e = RidgeJacobianInverse(domain)
+        assert is_cellwise_constant(e) == test
 
 
 def test_mappings_are_cellwise_constant_on_linear_affine_cells(affine_domains):
@@ -323,9 +333,7 @@ def test_coefficient_sometimes_cellwise_constant(domains_not_linear):
         "Discontinuous Lagrange", domains_not_linear.ufl_cell(), 0, (), identity_pullback, L2
     )
     d = domains_not_linear.ufl_cell().topological_dimension()
-    domain = Mesh(
-        FiniteElement("Lagrange", domains_not_linear.ufl_cell(), 1, (d,), identity_pullback, H1)
-    )
+    domain = Mesh(LagrangeElement(domains_not_linear.ufl_cell(), 1, (d,)))
     space = FunctionSpace(domain, V)
     e = Coefficient(space)
     assert is_cellwise_constant(e)
@@ -348,9 +356,7 @@ def test_coefficient_mostly_not_cellwise_constant(domains_not_linear):
         "Discontinuous Lagrange", domains_not_linear.ufl_cell(), 1, (), identity_pullback, L2
     )
     d = domains_not_linear.ufl_cell().topological_dimension()
-    domain = Mesh(
-        FiniteElement("Lagrange", domains_not_linear.ufl_cell(), 1, (d,), identity_pullback, H1)
-    )
+    domain = Mesh(LagrangeElement(domains_not_linear.ufl_cell(), 1, (d,)))
     space = FunctionSpace(domain, V)
     e = Coefficient(space)
     assert not is_cellwise_constant(e)
