@@ -8,6 +8,8 @@ Next look at the TODO markers below for places to edit.
 
 import warnings
 
+from utils import LagrangeElement
+
 from ufl import (
     Coefficient,
     FunctionSpace,
@@ -25,9 +27,6 @@ from ufl import (
     triangle,
 )
 from ufl.classes import FixedIndex, FormArgument, Grad, Indexed, ListTensor, Zero
-from ufl.finiteelement import FiniteElement
-from ufl.pullback import identity_pullback
-from ufl.sobolevspace import H1
 from ufl.tensors import as_scalar, unit_indexed_tensor, unwrap_list_tensor
 
 
@@ -55,15 +54,15 @@ class MockForwardAD:
             (o,) = o.ufl_operands
             ngrads += 1
         if not isinstance(o, FormArgument):
-            raise ValueError("Expecting gradient of a FormArgument, not %s" % repr(o))
+            raise ValueError(f"Expecting gradient of a FormArgument, not {o!r}")
 
         def apply_grads(f):
             if not isinstance(f, FormArgument):
-                print(("," * 60))
+                print("," * 60)
                 print(f)
                 print(o)
                 print(g)
-                print(("," * 60))
+                print("," * 60)
                 raise ValueError("What?")
             for i in range(ngrads):
                 f = Grad(f)
@@ -164,7 +163,7 @@ class MockForwardAD:
                 if self._cd._data:
                     # TODO: Make it possible to silence this message in particular?
                     #       It may be good to have for debugging...
-                    warnings.warn("Assuming d{%s}/d{%s} = 0." % (o, self._w))
+                    warnings.warn(f"Assuming d{{{o}}}/d{{{self._w}}} = 0.")
             else:
                 # Make sure we have a tuple to match the self._v tuple
                 if not isinstance(oprimes, tuple):
@@ -195,9 +194,9 @@ class MockForwardAD:
 
 
 def test_unit_tensor(self):
-    E2_1, ii = unit_indexed_tensor((2,), (1,))
-    E3_1, ii = unit_indexed_tensor((3,), (1,))
-    E22_10, ii = unit_indexed_tensor((2, 2), (1, 0))
+    _, _ = unit_indexed_tensor((2,), (1,))
+    _, _ = unit_indexed_tensor((3,), (1,))
+    _, _ = unit_indexed_tensor((2, 2), (1, 0))
     # TODO: Evaluate and assert values
 
 
@@ -236,8 +235,8 @@ def test_unwrap_list_tensor(self):
 
 
 def test__forward_coefficient_ad__grad_of_scalar_coefficient(self):
-    U = FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    U = LagrangeElement(triangle, 1)
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, U)
     u = Coefficient(space)
     du = TestFunction(space)
@@ -262,8 +261,8 @@ def test__forward_coefficient_ad__grad_of_scalar_coefficient(self):
 
 
 def test__forward_coefficient_ad__grad_of_vector_coefficient(self):
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, V)
     v = Coefficient(space)
     dv = TestFunction(space)
@@ -288,8 +287,8 @@ def test__forward_coefficient_ad__grad_of_vector_coefficient(self):
 
 
 def test__forward_coefficient_ad__grad_of_vector_coefficient__with_component_variation(self):
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, V)
     v = Coefficient(space)
     dv = TestFunction(space)
@@ -348,8 +347,8 @@ def test__forward_coefficient_ad__grad_of_vector_coefficient__with_component_var
 def test__forward_coefficient_ad__grad_of_vector_coefficient__with_component_variation_in_list(
     self,
 ):
-    V = FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    V = LagrangeElement(triangle, 1, (2,))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, V)
     v = Coefficient(space)
     dv = TestFunction(space)
@@ -406,8 +405,8 @@ def test__forward_coefficient_ad__grad_of_vector_coefficient__with_component_var
 
 
 def test__forward_coefficient_ad__grad_of_tensor_coefficient(self):
-    W = FiniteElement("Lagrange", triangle, 1, (2, 2), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    W = LagrangeElement(triangle, 1, (2, 2))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, W)
     w = Coefficient(space)
     dw = TestFunction(space)
@@ -432,8 +431,8 @@ def test__forward_coefficient_ad__grad_of_tensor_coefficient(self):
 
 
 def test__forward_coefficient_ad__grad_of_tensor_coefficient__with_component_variation(self):
-    W = FiniteElement("Lagrange", triangle, 1, (2, 2), identity_pullback, H1)
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    W = LagrangeElement(triangle, 1, (2, 2))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     space = FunctionSpace(domain, W)
     w = Coefficient(space)
     dw = TestFunction(space)
