@@ -1,8 +1,9 @@
 __authors__ = "Martin Sandve Alnæs"
 __date__ = "2009-03-14 -- 2009-03-14"
 
+from utils import FiniteElement, LagrangeElement, MixedElement, SymmetricElement
+
 from ufl import Coefficient, FunctionSpace, Mesh, TestFunction, as_vector, product, split, triangle
-from ufl.finiteelement import FiniteElement, MixedElement, SymmetricElement
 from ufl.pullback import identity_pullback
 from ufl.sobolevspace import H1
 
@@ -10,8 +11,8 @@ from ufl.sobolevspace import H1
 def test_split(self):
     cell = triangle
     d = cell.topological_dimension()
-    domain = Mesh(FiniteElement("Lagrange", cell, 1, (d,), identity_pullback, H1))
-    f = FiniteElement("Lagrange", cell, 1, (), identity_pullback, H1)
+    domain = Mesh(LagrangeElement(cell, 1, (d,)))
+    f = LagrangeElement(cell, 1)
     v = FiniteElement(
         "Lagrange", cell, 1, (d,), identity_pullback, H1, sub_elements=[f for _ in range(d)]
     )
