@@ -1,3 +1,5 @@
+from utils import FiniteElement, LagrangeElement
+
 from ufl import (
     CellDiameter,
     CellVolume,
@@ -17,7 +19,6 @@ from ufl import (
     tetrahedron,
     triangle,
 )
-from ufl.finiteelement import FiniteElement
 from ufl.pullback import identity_pullback
 from ufl.sobolevspace import H1
 
@@ -31,7 +32,7 @@ def test_str_float_value(self):
 
 
 def test_str_zero(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     x = SpatialCoordinate(domain)
     assert str(as_ufl(0)) == "0"
     assert str(0 * x) == "0 (shape (2,))"
@@ -44,45 +45,41 @@ def test_str_index(self):
 
 
 def test_str_coordinate(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(SpatialCoordinate(domain)) == "x"
     assert str(SpatialCoordinate(domain)[0]) == "x[0]"
 
 
 def test_str_normal(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(FacetNormal(domain)) == "n"
     assert str(FacetNormal(domain)[0]) == "n[0]"
 
 
 def test_str_circumradius(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(Circumradius(domain)) == "circumradius"
 
 
 def test_str_diameter(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(CellDiameter(domain)) == "diameter"
 
 
 def test_str_facetarea(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(FacetArea(domain)) == "facetarea"
 
 
 def test_str_volume(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     assert str(CellVolume(domain)) == "volume"
 
 
 def test_str_scalar_argument(self):
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
-    v = TestFunction(
-        FunctionSpace(domain, FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1))
-    )
-    u = TrialFunction(
-        FunctionSpace(domain, FiniteElement("Lagrange", triangle, 1, (), identity_pullback, H1))
-    )
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
+    v = TestFunction(FunctionSpace(domain, LagrangeElement(triangle, 1)))
+    u = TrialFunction(FunctionSpace(domain, LagrangeElement(triangle, 1)))
     assert str(v) == "v_0"
     assert str(u) == "v_1"
 
@@ -95,38 +92,38 @@ def test_str_scalar_argument(self):
 
 
 def test_str_list_vector():
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
     x, y, z = SpatialCoordinate(domain)
-    v = as_vector((x, y, z))
-    assert str(v) == ("[%s, %s, %s]" % (x, y, z))
+    v = as_vector((z, y, x))
+    assert str(v) == (f"[{z}, {y}, {x}]")
 
 
 def test_str_list_vector_with_zero():
-    domain = Mesh(FiniteElement("Lagrange", tetrahedron, 1, (3,), identity_pullback, H1))
-    x, y, z = SpatialCoordinate(domain)
+    domain = Mesh(LagrangeElement(tetrahedron, 1, (3,)))
+    x, _y, _z = SpatialCoordinate(domain)
     v = as_vector((x, 0, 0))
-    assert str(v) == ("[%s, 0, 0]" % (x,))
+    assert str(v) == (f"[{x}, 0, 0]")
 
 
 def test_str_list_matrix():
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     x, y = SpatialCoordinate(domain)
     v = as_matrix(((2 * x, 3 * y), (4 * x, 5 * y)))
     a = str(2 * x)
     b = str(3 * y)
     c = str(4 * x)
     d = str(5 * y)
-    assert str(v) == ("[\n  [%s, %s],\n  [%s, %s]\n]" % (a, b, c, d))
+    assert str(v) == (f"[\n  [{a}, {b}],\n  [{c}, {d}]\n]")
 
 
 def test_str_list_matrix_with_zero():
-    domain = Mesh(FiniteElement("Lagrange", triangle, 1, (2,), identity_pullback, H1))
+    domain = Mesh(LagrangeElement(triangle, 1, (2,)))
     x, y = SpatialCoordinate(domain)
     v = as_matrix(((2 * x, 3 * y), (0, 0)))
     a = str(2 * x)
     b = str(3 * y)
     c = str(as_vector((0, 0)))
-    assert str(v) == ("[\n  [%s, %s],\n%s\n]" % (a, b, c))
+    assert str(v) == (f"[\n  [{a}, {b}],\n{c}\n]")
 
 
 # FIXME: Add more tests for tensors collapsing
@@ -135,8 +132,5 @@ def test_str_list_matrix_with_zero():
 
 def test_str_element():
     elem = FiniteElement("Q", quadrilateral, 1, (), identity_pullback, H1)
-    assert (
-        repr(elem)
-        == 'ufl.finiteelement.FiniteElement("Q", quadrilateral, 1, (), IdentityPullback(), H1)'
-    )
+    assert repr(elem) == 'utils.FiniteElement("Q", quadrilateral, 1, (), IdentityPullback(), H1)'
     assert str(elem) == "<Q1 on a quadrilateral>"
