@@ -28,13 +28,13 @@ class Adjoint(BaseForm):
     """
 
     __slots__ = (
-        "_form",
-        "_repr",
         "_arguments",
         "_coefficients",
         "_domains",
-        "ufl_operands",
+        "_form",
         "_hash",
+        "_repr",
+        "ufl_operands",
     )
 
     def __new__(cls, *args, **kw):
@@ -49,7 +49,7 @@ class Adjoint(BaseForm):
             return form._form
         elif isinstance(form, FormSum):
             # Adjoint distributes over sums
-            return FormSum(*[(Adjoint(component), 1) for component in form.components()])
+            return FormSum(*((Adjoint(c), w) for c, w in zip(form.components(), form.weights())))
         elif isinstance(form, Coargument):
             # The adjoint of a coargument `c: V* -> V*` is the identity
             # matrix mapping from V to V (i.e. V x V* -> R).
@@ -62,7 +62,7 @@ class Adjoint(BaseForm):
             # subclasses of `ufl.Coargument`.
             return primal_arg
 
-        return super(Adjoint, cls).__new__(cls)
+        return super().__new__(cls)
 
     def __init__(self, form):
         """Initialise."""
@@ -75,7 +75,7 @@ class Adjoint(BaseForm):
         self.ufl_operands = (self._form,)
         self._domains = None
         self._hash = None
-        self._repr = "Adjoint(%s)" % repr(self._form)
+        self._repr = f"Adjoint({self._form!r})"
 
     def ufl_function_spaces(self):
         """Get the tuple of function spaces of the underlying form."""
