@@ -13,6 +13,7 @@ from ufl.core.expr import ufl_err_str
 from ufl.core.operator import Operator
 from ufl.core.ufl_type import ufl_type
 from ufl.index_combination_utils import merge_unique_indices
+from ufl.indexed import Indexed
 from ufl.precedence import parstr
 from ufl.sorting import sorted_expr
 
@@ -89,6 +90,11 @@ class Sum(Operator):
         """Initialise."""
         Operator.__init__(self)
 
+    def _simplify_indexed(self, multiindex):
+        """Return a simplified Expr used in the constructor of Indexed(self, multiindex)."""
+        a, b = self.ufl_operands
+        return Sum(Indexed(a, multiindex), Indexed(b, multiindex))
+
     def evaluate(self, x, mapping, component, index_values):
         """Evaluate."""
         return sum(o.evaluate(x, mapping, component, index_values) for o in self.ufl_operands)
@@ -253,12 +259,7 @@ class Division(Operator):
         a, b = self.ufl_operands
         a = a.evaluate(x, mapping, component, index_values)
         b = b.evaluate(x, mapping, component, index_values)
-        # Avoiding integer division by casting to float
-        try:
-            e = float(a) / float(b)
-        except TypeError:
-            e = complex(a) / complex(b)
-        return e
+        return a / b
 
     def __str__(self):
         """Format as a string."""
@@ -338,7 +339,7 @@ class Abs(Operator):
         a = as_ufl(a)
 
         # Simplification
-        if isinstance(a, (Zero, Abs)):
+        if isinstance(a, Zero | Abs):
             return a
         if isinstance(a, Conj):
             return Abs(a.ufl_operands[0])
@@ -373,7 +374,7 @@ class Conj(Operator):
         a = as_ufl(a)
 
         # Simplification
-        if isinstance(a, (Abs, Real, Imag, Zero)):
+        if isinstance(a, Abs | Real | Imag | Zero):
             return a
         if isinstance(a, Conj):
             return a.ufl_operands[0]
@@ -447,7 +448,7 @@ class Imag(Operator):
         # Simplification
         if isinstance(a, Zero):
             return a
-        if isinstance(a, (Real, Imag, Abs)):
+        if isinstance(a, Real | Imag | Abs):
             return Zero(a.ufl_shape, a.ufl_free_indices, a.ufl_index_dimensions)
         if isinstance(a, ScalarValue):
             return as_ufl(a.imag())
