@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Front-end for AD routines."""
 
 # Copyright (C) 2008-2016 Martin Sandve Alnæs
@@ -11,7 +10,6 @@
 
 import warnings
 
-from ufl.adjoint import Adjoint
 from ufl.algorithms.apply_algebra_lowering import apply_algebra_lowering
 from ufl.algorithms.apply_derivatives import apply_derivatives
 
@@ -29,13 +27,6 @@ def expand_derivatives(form, **kwargs):
     if kwargs:
         warnings("Deprecation: expand_derivatives no longer takes any keyword arguments")
 
-    if isinstance(form, Adjoint):
-        dform = expand_derivatives(form._form)
-        if dform == 0:
-            return dform
-        # Adjoint is taken on a 3-form which can't happen
-        raise NotImplementedError('Adjoint derivative is not supported.')
-
     # Lower abstractions for tensor-algebra types into index notation
     form = apply_algebra_lowering(form)
 
