@@ -16,7 +16,7 @@ from ufl.utils.counted import Counted
 __all_classes__ = ["IndexBase", "FixedIndex", "Index"]
 
 
-class IndexBase(object):
+class IndexBase:
     """Base class for all indices."""
 
     __slots__ = ()
@@ -28,9 +28,9 @@ class IndexBase(object):
 class FixedIndex(IndexBase):
     """UFL value: An index with a specific value assigned."""
 
-    __slots__ = ("_value", "_hash")
+    __slots__ = ("_hash", "_value")
 
-    _cache = {}
+    _cache: dict[int, "FixedIndex"] = {}
 
     def __getnewargs__(self):
         """Get new args."""
@@ -62,7 +62,7 @@ class FixedIndex(IndexBase):
 
     def __eq__(self, other):
         """Check equality."""
-        return isinstance(other, FixedIndex) and (self._value == other._value)
+        return isinstance(other, FixedIndex | int) and int(self) == int(other)
 
     def __int__(self):
         """Convert to int."""
@@ -116,7 +116,7 @@ class MultiIndex(Terminal):
 
     __slots__ = ("_indices",)
 
-    _cache = {}
+    _cache: dict[tuple[int], "MultiIndex"] = {}
 
     def __getnewargs__(self):
         """Get new args."""
@@ -162,7 +162,7 @@ class MultiIndex(Terminal):
 
     def _ufl_compute_hash_(self):
         """Compute UFL hash."""
-        return hash(("MultiIndex",) + tuple(hash(ind) for ind in self._indices))
+        return hash(("MultiIndex", *map(hash, self._indices)))
 
     def __eq__(self, other):
         """Check equality."""
@@ -236,7 +236,7 @@ class MultiIndex(Terminal):
 
     def __str__(self):
         """Format as a string."""
-        return ", ".join(str(i) for i in self._indices)
+        return ", ".join(map(str, self._indices))
 
     def __repr__(self):
         """Return representation."""
