@@ -24,6 +24,7 @@ from ufl.algorithms import (
     replace,  # noqa: F401
 )
 from ufl.argument import Argument
+from ufl.cell import Cell
 from ufl.coefficient import Coefficient, Cofunction
 from ufl.constantvalue import as_ufl, is_true_ufl_scalar
 from ufl.core.base_form_operator import BaseFormOperator
@@ -38,18 +39,87 @@ from ufl.differentiation import (
     CoordinateDerivative,
 )
 from ufl.exprcontainers import ExprList, ExprMapping
-from ufl.finiteelement import MixedElement
+from ufl.finiteelement import AbstractFiniteElement
 from ufl.form import BaseForm, Form, FormSum, ZeroBaseForm, as_form
 from ufl.functionspace import FunctionSpace
 from ufl.geometry import SpatialCoordinate
 from ufl.indexed import Indexed
+from ufl.pullback import AbstractPullback, IdentityPullback, MixedPullback
+from ufl.sobolevspace import SobolevSpace
 from ufl.sorting import sorted_expr
 from ufl.split_functions import split
 from ufl.tensors import ListTensor, as_tensor
 from ufl.variable import Variable
 
 
-def extract_blocks(form, i=None, j=None):
+class _MixedElement(AbstractFiniteElement):
+    """A mixed element."""
+
+    def __init__(self, sub_elements):
+        """Initialise a mixed element."""
+        for e in sub_elements:
+            assert e.cell == sub_elements[0].cell
+        if all(isinstance(e.pullback, IdentityPullback) for e in sub_elements):
+            self._pullback = IdentityPullback()
+        else:
+            self._pullback = MixedPullback(self)
+        self._sub_elements = sub_elements
+
+    def __repr__(self) -> str:
+        """Format as string for evaluation as Python object."""
+        return f"ufl.formoperators._MixedElement({self._sub_elements!r})"
+
+    def __str__(self) -> str:
+        """Format as string for nice printing."""
+        return f"<MixedElement with {len(self._sub_elements)} sub-element(s)>"
+
+    def __hash__(self) -> int:
+        """Return a hash."""
+        return hash(f"{self!r}")
+
+    def __eq__(self, other) -> bool:
+        """Check if this element is equal to another element."""
+        return type(self) is type(other) and repr(self) == repr(other)
+
+    @property
+    def sobolev_space(self) -> SobolevSpace:
+        """Return the underlying Sobolev space."""
+        return max(e.sobolev_space for e in self._sub_elements)
+
+    @property
+    def pullback(self) -> AbstractPullback:
+        """Return the pullback for this element."""
+        return self._pullback
+
+    @property
+    def embedded_superdegree(self) -> int | None:
+        """Degree of the minimum degree Lagrange space that spans this element."""
+        return max(e.embedded_superdegree for e in self._sub_elements)
+
+    @property
+    def embedded_subdegree(self) -> int:
+        """Degree of the maximum degree Lagrange space that is spanned by this element."""
+        return min(e.embedded_subdegree for e in self._sub_elements)
+
+    @property
+    def cell(self) -> Cell:
+        """Return the cell of the finite element."""
+        return self._sub_elements[0].cell
+
+    @property
+    def reference_value_shape(self) -> tuple[int, ...]:
+        """Return the shape of the value space on the reference cell."""
+        return (sum(e.reference_value_size for e in self._sub_elements),)
+
+    @property
+    def sub_elements(self) -> list:
+        """Return list of sub-elements."""
+        return self._sub_elements
+
+
+def extract_blocks(
+    form: Form, i: int | None = None, j: int | None = None, replace_argument: bool = True
+):
     """Extract blocks.
 
     Given a linear or bilinear form on a mixed space,
@@ -60,7 +130,7 @@ def extract_blocks(form, i=None, j=None):
        extract_blocks(a, 0, 0) -> inner(grad(u), grad(v))*dx
        extract_blocks(a) -> [inner(grad(u), grad(v))*dx, div(v)*p*dx, div(u)*q*dx, 0]
     """
-    return formsplitter.extract_blocks(form, i, j)
+    return formsplitter.extract_blocks(form, i, j, replace_argument=replace_argument)
 
 
 def lhs(form):
@@ -126,7 +196,7 @@ def action(form, coefficient=None, derivatives_expanded=None):
         isinstance(coefficient, BaseFormOperator) and len(coefficient.arguments()) == 1
     )
     # Can't expand derivatives on objects that are not Form or Expr (e.g. Matrix)
-    if isinstance(form, (Form, BaseFormOperator)) and is_coefficient_valid:
+    if isinstance(form, Form | BaseFormOperator) and is_coefficient_valid:
         if not derivatives_expanded:
             # For external operators differentiation may turn a Form into a FormSum
             form = expand_derivatives(form)
@@ -200,7 +270,7 @@ def set_list_item(li, i, v):
 def _handle_derivative_arguments(form, coefficient, argument):
     """Handle derivative arguments."""
     # Wrap single coefficient in tuple for uniform treatment below
-    if isinstance(coefficient, (list, tuple, ListTensor)):
+    if isinstance(coefficient, list | tuple | ListTensor):
         coefficients = tuple(coefficient)
     else:
         coefficients = (coefficient,)
@@ -208,7 +278,7 @@ def _handle_derivative_arguments(form, coefficient, argument):
     if argument is None:
         # Try to create argument if not provided
         if not all(
-            isinstance(c, (Coefficient, Cofunction, BaseFormOperator)) for c in coefficients
+            isinstance(c, Coefficient | Cofunction | BaseFormOperator) for c in coefficients
         ):
             raise ValueError(
                 "Can only create arguments automatically for non-indexed coefficients."
@@ -242,12 +312,12 @@ def _handle_derivative_arguments(form, coefficient, argument):
             domains = [fs.ufl_domain() for fs in function_spaces]
             elements = [fs.ufl_element() for fs in function_spaces]
             assert all(fs.ufl_domain() == domains[0] for fs in function_spaces)
-            elm = MixedElement(elements)
+            elm = _MixedElement(elements)
             fs = FunctionSpace(domains[0], elm)
             arguments = split(Argument(fs, number, part))
     else:
         # Wrap single argument in tuple for uniform treatment below
-        if isinstance(argument, (list, tuple)):
+        if isinstance(argument, list | tuple):
             arguments = tuple(argument)
         else:
             n = len(coefficients)
@@ -264,7 +334,7 @@ def _handle_derivative_arguments(form, coefficient, argument):
     for c, a in zip(coefficients, arguments):
         if c.ufl_shape != a.ufl_shape:
             raise ValueError("Coefficient and argument shapes do not match!")
-        if isinstance(c, (Coefficient, Cofunction, BaseFormOperator, SpatialCoordinate)):
+        if isinstance(c, Coefficient | Cofunction | BaseFormOperator | SpatialCoordinate):
             m[c] = a
         else:
             if not isinstance(c, Indexed):
@@ -319,8 +389,8 @@ def derivative(form, coefficient, argument=None, coefficient_derivatives=None):
         # Distribute derivative over FormSum components
         return FormSum(
             *[
-                (derivative(component, coefficient, argument, coefficient_derivatives), 1)
-                for component in form.components()
+                (derivative(component, coefficient, argument, coefficient_derivatives), w)
+                for component, w in zip(form.components(), form.weights())
             ]
         )
     elif isinstance(form, Adjoint):
