@@ -7,7 +7,6 @@
 
 from __future__ import annotations
 
-import typing
 from abc import ABC, abstractmethod, abstractproperty
 from typing import TYPE_CHECKING
 
@@ -15,7 +14,7 @@ import numpy as np
 
 from ufl.core.expr import Expr
 from ufl.core.multiindex import indices
-from ufl.domain import extract_unique_domain
+from ufl.domain import AbstractDomain, MeshSequence, extract_unique_domain
 from ufl.functionspace import FunctionSpace
 from ufl.tensors import as_tensor
 
@@ -31,6 +30,7 @@ __all_classes__ = [
     "L2Piola",
     "DoubleContravariantPiola",
     "DoubleCovariantPiola",
+    "CovariantContravariantPiola",
     "MixedPullback",
     "SymmetricPullback",
     "PhysicalPullback",
@@ -53,7 +53,7 @@ class AbstractPullback(ABC):
         """Return a representation of the object."""
 
     @abstractmethod
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -68,11 +68,12 @@ class AbstractPullback(ABC):
     def is_identity(self) -> bool:
         """Is this pull back the identity (or the identity applied to mutliple components)."""
 
-    def apply(self, expr: Expr) -> Expr:
+    def apply(self, expr: Expr, domain: AbstractDomain | None = None) -> Expr:
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
@@ -91,17 +92,18 @@ class IdentityPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return True
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         return expr
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -126,17 +128,18 @@ class ContravariantPiola(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return False
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         from ufl.classes import Jacobian, JacobianDeterminant
 
-        domain = extract_unique_domain(expr)
+        domain = domain or extract_unique_domain(expr)
         J = Jacobian(domain)
         detJ = JacobianDeterminant(J)
         transform = (1.0 / detJ) * J
@@ -145,7 +148,7 @@ class ContravariantPiola(AbstractPullback):
         kj = (*k, j)
         return as_tensor(transform[i, j] * expr[kj], (*k, i))
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -156,7 +159,7 @@ class ContravariantPiola(AbstractPullback):
             The value shape when the pull back is applied to the given element
         """
         gdim = domain.geometric_dimension()
-        return (gdim,) + element.reference_value_shape[1:]
+        return element.reference_value_shape[:-1] + (gdim,)
 
 
 class CovariantPiola(AbstractPullback):
@@ -171,24 +174,25 @@ class CovariantPiola(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return False
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         from ufl.classes import JacobianInverse
 
-        domain = extract_unique_domain(expr)
+        domain = domain or extract_unique_domain(expr)
         K = JacobianInverse(domain)
         # Apply transform "row-wise" to TensorElement(PiolaMapped, ...)
         *k, i, j = indices(len(expr.ufl_shape) + 1)
         kj = (*k, j)
         return as_tensor(K[j, i] * expr[kj], (*k, i))
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -199,7 +203,7 @@ class CovariantPiola(AbstractPullback):
             The value shape when the pull back is applied to the given element
         """
         gdim = domain.geometric_dimension()
-        return (gdim,) + element.reference_value_shape[1:]
+        return element.reference_value_shape[:-1] + (gdim,)
 
 
 class L2Piola(AbstractPullback):
@@ -214,21 +218,22 @@ class L2Piola(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return False
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         from ufl.classes import JacobianDeterminant
 
-        domain = extract_unique_domain(expr)
+        domain = domain or extract_unique_domain(expr)
         detJ = JacobianDeterminant(domain)
         return expr / detJ
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -253,17 +258,18 @@ class DoubleContravariantPiola(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return False
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         from ufl.classes import Jacobian, JacobianDeterminant
 
-        domain = extract_unique_domain(expr)
+        domain = domain or extract_unique_domain(expr)
         J = Jacobian(domain)
         detJ = JacobianDeterminant(J)
         # Apply transform "row-wise" to TensorElement(PiolaMapped, ...)
@@ -271,7 +277,7 @@ class DoubleContravariantPiola(AbstractPullback):
         kmn = (*k, m, n)
         return as_tensor((1.0 / detJ) ** 2 * J[i, m] * expr[kmn] * J[j, n], (*k, i, j))
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -282,7 +288,7 @@ class DoubleContravariantPiola(AbstractPullback):
             The value shape when the pull back is applied to the given element
         """
         gdim = domain.geometric_dimension()
-        return (gdim, gdim)
+        return element.reference_value_shape[:-2] + (gdim, gdim)
 
 
 class DoubleCovariantPiola(AbstractPullback):
@@ -297,24 +303,25 @@ class DoubleCovariantPiola(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return False
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         from ufl.classes import JacobianInverse
 
-        domain = extract_unique_domain(expr)
+        domain = domain or extract_unique_domain(expr)
         K = JacobianInverse(domain)
         # Apply transform "row-wise" to TensorElement(PiolaMapped, ...)
         *k, i, j, m, n = indices(len(expr.ufl_shape) + 2)
         kmn = (*k, m, n)
         return as_tensor(K[m, i] * expr[kmn] * K[n, j], (*k, i, j))
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -325,7 +332,53 @@ class DoubleCovariantPiola(AbstractPullback):
             The value shape when the pull back is applied to the given element
         """
         gdim = domain.geometric_dimension()
-        return (gdim, gdim)
+        return element.reference_value_shape[:-2] + (gdim, gdim)
+
+
+class CovariantContravariantPiola(AbstractPullback):
+    """The covariant contravariant Piola pull back."""
+
+    def __repr__(self) -> str:
+        """Return a representation of the object."""
+        return "CovariantContravariantPiola()"
+
+    @property
+    def is_identity(self) -> bool:
+        """Is this pull back the identity (or the identity applied to mutliple components)."""
+        return False
+
+    def apply(self, expr, domain=None):
+        """Apply the pull back.
+
+        Args:
+            expr: A function on a physical cell
+            domain: The domain on which the function is defined
+
+        Returns: The function pulled back to the reference cell
+        """
+        from ufl.classes import Jacobian, JacobianDeterminant, JacobianInverse
+
+        domain = domain or extract_unique_domain(expr)
+        J = Jacobian(domain)
+        detJ = JacobianDeterminant(J)
+        K = JacobianInverse(domain)
+        # Apply transform "row-wise" to TensorElement(PiolaMapped, ...)
+        *k, i, j, m, n = indices(len(expr.ufl_shape) + 2)
+        kmn = (*k, m, n)
+        return as_tensor((1.0 / detJ) * K[m, i] * expr[kmn] * J[j, n], (*k, i, j))
+
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
+        """Get the physical value shape when this pull back is applied to an element.
+
+        Args:
+            element: The element that the pull back is applied to
+            domain: The domain
+
+        Returns:
+            The value shape when the pull back is applied to the given element
+        """
+        gdim = domain.geometric_dimension()
+        return element.reference_value_shape[:-2] + (gdim, gdim)
 
 
 class MixedPullback(AbstractPullback):
@@ -348,31 +401,37 @@ class MixedPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return all(e.pullback.is_identity for e in self._element.sub_elements)
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
-        domain = extract_unique_domain(expr)
-        space = FunctionSpace(domain, self._element)
         rflat = [expr[idx] for idx in np.ndindex(expr.ufl_shape)]
         g_components = []
         offset = 0
         # For each unique piece in reference space, apply the appropriate pullback
-        for subelem in self._element.sub_elements:
+        domain = domain or extract_unique_domain(expr, expand_mesh_sequence=False)
+        if isinstance(domain, MeshSequence):
+            if len(domain) != self._element.num_sub_elements:
+                raise ValueError(f"""num. component meshes ({len(domain)}) !=
+                    num. sub elements ({self._element.num_sub_elements})""")
+        for i, subelem in enumerate(self._element.sub_elements):
             rsub = as_tensor(
                 np.asarray(rflat[offset : offset + subelem.reference_value_size]).reshape(
                     subelem.reference_value_shape
                 )
             )
-            rmapped = subelem.pullback.apply(rsub)
+            subdomain = domain[i] if isinstance(domain, MeshSequence) else None
+            rmapped = subelem.pullback.apply(rsub, domain=subdomain)
             # Flatten into the pulled back expression for the whole thing
             g_components.extend([rmapped[idx] for idx in np.ndindex(rmapped.ufl_shape)])
             offset += subelem.reference_value_size
         # And reshape appropriately
+        space = FunctionSpace(domain, self._element)
         f = as_tensor(np.asarray(g_components).reshape(space.value_shape))
         if f.ufl_shape != space.value_shape:
             raise ValueError(
@@ -381,7 +440,7 @@ class MixedPullback(AbstractPullback):
             )
         return f
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -392,16 +451,17 @@ class MixedPullback(AbstractPullback):
             The value shape when the pull back is applied to the given element
         """
         assert element == self._element
-        dim = sum(FunctionSpace(domain, e).value_size for e in self._element.sub_elements)
+        domains = domain.iterable_like(element)
+        dim = sum(
+            FunctionSpace(d, e).value_size for d, e in zip(domains, self._element.sub_elements)
+        )
         return (dim,)
 
 
 class SymmetricPullback(AbstractPullback):
     """Pull back for an element with symmetry."""
 
-    def __init__(
-        self, element: _AbstractFiniteElement, symmetry: typing.Dict[typing.tuple[int, ...], int]
-    ):
+    def __init__(self, element: _AbstractFiniteElement, symmetry: dict[tuple[int, ...], int]):
         """Initalise.
 
         Args:
@@ -427,11 +487,12 @@ class SymmetricPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return all(e.pullback.is_identity for e in self._element.sub_elements)
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
@@ -443,6 +504,11 @@ class SymmetricPullback(AbstractPullback):
         for subelem in self._element.sub_elements:
             offsets.append(offsets[-1] + subelem.reference_value_size)
         # For each unique piece in reference space, apply the appropriate pullback
+        domain = domain or extract_unique_domain(expr, expand_mesh_sequence=False)
+        if isinstance(domain, MeshSequence):
+            if len(domain) != self._element.num_sub_elements:
+                raise ValueError(f"""num. component meshes ({len(domain)}) !=
+                    num. sub elements ({self._element.num_sub_elements})""")
         for component in np.ndindex(self._block_shape):
             i = self._symmetry[component]
             subelem = self._element.sub_elements[i]
@@ -451,7 +517,8 @@ class SymmetricPullback(AbstractPullback):
                     subelem.reference_value_shape
                 )
             )
-            rmapped = subelem.pullback.apply(rsub)
+            subdomain = domain[i] if isinstance(domain, MeshSequence) else None
+            rmapped = subelem.pullback.apply(rsub, domain=subdomain)
             # Flatten into the pulled back expression for the whole thing
             g_components.extend([rmapped[idx] for idx in np.ndindex(rmapped.ufl_shape)])
         # And reshape appropriately
@@ -463,7 +530,7 @@ class SymmetricPullback(AbstractPullback):
             )
         return f
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -473,8 +540,10 @@ class SymmetricPullback(AbstractPullback):
         Returns:
             The value shape when the pull back is applied to the given element
         """
-        assert element == self._element
-        return tuple(i + 1 for i in max(self._symmetry.keys()))
+        assert isinstance(element, type(self._element))
+        subelem = element.sub_elements[0]
+        pvs = subelem.pullback.physical_value_shape(subelem, domain)
+        return tuple(i + 1 for i in max(self._symmetry.keys())) + pvs
 
 
 class PhysicalPullback(AbstractPullback):
@@ -492,17 +561,18 @@ class PhysicalPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return True
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         return expr
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -512,7 +582,7 @@ class PhysicalPullback(AbstractPullback):
         Returns:
             The value shape when the pull back is applied to the given element
         """
-        raise NotImplementedError()
+        return element.reference_value_shape
 
 
 class CustomPullback(AbstractPullback):
@@ -530,17 +600,18 @@ class CustomPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return True
 
-    def apply(self, expr):
+    def apply(self, expr, domain=None):
         """Apply the pull back.
 
         Args:
             expr: A function on a physical cell
+            domain: The domain on which the function is defined
 
         Returns: The function pulled back to the reference cell
         """
         return expr
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -550,9 +621,7 @@ class CustomPullback(AbstractPullback):
         Returns:
             The value shape when the pull back is applied to the given element
         """
-        if element.reference_value_shape == ():
-            return ()
-        raise NotImplementedError()
+        return element.reference_value_shape
 
 
 class UndefinedPullback(AbstractPullback):
@@ -570,7 +639,7 @@ class UndefinedPullback(AbstractPullback):
         """Is this pull back the identity (or the identity applied to mutliple components)."""
         return True
 
-    def physical_value_shape(self, element, domain) -> typing.Tuple[int, ...]:
+    def physical_value_shape(self, element, domain) -> tuple[int, ...]:
         """Get the physical value shape when this pull back is applied to an element on a domain.
 
         Args:
@@ -580,7 +649,7 @@ class UndefinedPullback(AbstractPullback):
         Returns:
             The value shape when the pull back is applied to the given element
         """
-        raise NotImplementedError()
+        return element.reference_value_shape
 
 
 identity_pullback = IdentityPullback()
@@ -589,6 +658,7 @@ contravariant_piola = ContravariantPiola()
 l2_piola = L2Piola()
 double_covariant_piola = DoubleCovariantPiola()
 double_contravariant_piola = DoubleContravariantPiola()
+covariant_contravariant_piola = CovariantContravariantPiola()
 physical_pullback = PhysicalPullback()
 custom_pullback = CustomPullback()
 undefined_pullback = UndefinedPullback()
