@@ -22,7 +22,7 @@ from ufl.indexed import Indexed
 class ListTensor(Operator):
     """Wraps a list of expressions into a tensor valued expression of one higher rank."""
 
-    __slots__ = ()
+    __slots__ = ("_initialised",)
 
     def __new__(cls, *expressions):
         """Create a new ListTensor."""
@@ -53,13 +53,56 @@ class ListTensor(Operator):
 
         # Simplify to Zero if possible
         if all(isinstance(e, Zero) for e in expressions):
-            shape = (len(expressions),) + sh
+            shape = (len(expressions), *sh)
             return Zero(shape, fi, fid)
 
-        return Operator.__new__(cls)
+        def sub(e, *indices):
+            for i in indices:
+                e = e.ufl_operands[i]
+            return e
+
+        def sub_equals(exprs, *indices):
+            sube0 = sub(exprs[0], *indices)
+            return all(sub(e, *indices) is sube0 for e in exprs[1:])
+
+        # Simplify [v[j,0], v[j,1], ...., v[j,k]] -> v[j,:]
+        if (
+            all(isinstance(e, Indexed) for e in expressions)
+            and sub(e0, 0).ufl_shape[-1] == len(expressions)
+            and sub_equals(expressions, 0)
+        ):
+            indices = [sub(e, 1).indices() for e in expressions]
+            try:
+                (j,) = set(i[:-1] for i in indices)
+                if all(i[-1] == k for k, i in enumerate(indices)):
+                    return sub(e0, 0) if j == () else sub(e0, 0)[(*j, slice(None))]
+            except ValueError:
+                pass
+        # Simplify [v[0,:], v[1,:], ..., v[k,:]] -> v
+        if (
+            all(
+                isinstance(e, ComponentTensor) and isinstance(sub(e, 0), Indexed)
+                for e in expressions
+            )
+            and sub(e0, 0, 0).ufl_shape[0] == len(expressions)
+            and sub_equals(expressions, 0, 0)
+        ):
+            indices = [sub(e, 0, 1).indices() for e in expressions]
+            if all(
+                i[0] == k and all(isinstance(subindex, Index) for subindex in i[1:])
+                for k, i in enumerate(indices)
+            ):
+                return sub(e0, 0, 0)
+
+        # Construct a new instance to be initialised
+        self = Operator.__new__(cls)
+        self._initialised = False
+        return self
 
     def __init__(self, *expressions):
         """Initialise."""
+        if self._initialised:
+            return
         Operator.__init__(self, expressions)
 
         # Checks
@@ -68,6 +111,7 @@ class ListTensor(Operator):
             raise ValueError(
                 "Can't combine subtensor expressions with different sets of free indices."
             )
+        self._initialised = True
 
     @property
     def ufl_shape(self):
@@ -88,6 +132,14 @@ class ListTensor(Operator):
         else:
             return a.evaluate(x, mapping, component, index_values)
 
+    def _simplify_indexed(self, multiindex):
+        """Return a simplified Expr used in the constructor of Indexed(self, multiindex)."""
+        k = multiindex[0]
+        if isinstance(k, FixedIndex):
+            sub = self.ufl_operands[int(k)]
+            return Indexed(sub, MultiIndex(multiindex[1:]))
+        return Operator._simplify_indexed(self, multiindex)
+
     def __getitem__(self, key):
         """Get an item."""
         origkey = key
@@ -96,8 +148,10 @@ class ListTensor(Operator):
             key = key.indices()
         if not isinstance(key, tuple):
             key = (key,)
+        if len(key) == 0:
+            return self
         k = key[0]
-        if isinstance(k, (int, FixedIndex)):
+        if isinstance(k, int | FixedIndex):
             sub = self.ufl_operands[int(k)]
             return sub if len(key) == 1 else sub[key[1:]]
 
@@ -116,10 +170,10 @@ class ListTensor(Operator):
                     else:
                         substrings.append(str(e))
                 s = (",\n" + ind).join(substrings)
-                return "%s[\n%s%s\n%s]" % (ind, ind, s, ind)
+                return f"{ind}[\n{ind}{s}\n{ind}]"
             else:
-                s = ", ".join(str(e) for e in expressions)
-                return "%s[%s]" % (ind, s)
+                s = ", ".join(map(str, expressions))
+                return f"{ind}[{s}]"
 
         return substring(self.ufl_operands, 0)
 
@@ -128,11 +182,11 @@ class ListTensor(Operator):
 class ComponentTensor(Operator):
     """Maps the free indices of a scalar valued expression to tensor axes."""
 
-    __slots__ = ("ufl_shape", "ufl_free_indices", "ufl_index_dimensions")
+    __slots__ = ("_initialised", "ufl_free_indices", "ufl_index_dimensions", "ufl_shape")
 
     def __new__(cls, expression, indices):
         """Create a new ComponentTensor."""
-        # Simplify
+        # Zero-simplify
         if isinstance(expression, Zero):
             fi, fid, sh = remove_indices(
                 expression.ufl_free_indices,
@@ -141,11 +195,21 @@ class ComponentTensor(Operator):
             )
             return Zero(sh, fi, fid)
 
-        # Construct
-        return Operator.__new__(cls)
+        # Special case for simplification as_tensor(A[ii], ii) -> A
+        if isinstance(expression, Indexed):
+            A, ii = expression.ufl_operands
+            if indices == ii:
+                return A
+
+        # Construct a new instance to be initialised
+        self = Operator.__new__(cls)
+        self._initialised = False
+        return self
 
     def __init__(self, expression, indices):
         """Initialise."""
+        if self._initialised:
+            return
         if not isinstance(expression, Expr):
             raise ValueError("Expecting ufl expression.")
         if expression.ufl_shape != ():
@@ -165,15 +229,21 @@ class ComponentTensor(Operator):
         self.ufl_free_indices = fi
         self.ufl_index_dimensions = fid
         self.ufl_shape = sh
+        self._initialised = True
 
-    def _ufl_expr_reconstruct_(self, expressions, indices):
-        """Reconstruct."""
-        # Special case for simplification as_tensor(A[ii], ii) -> A
-        if isinstance(expressions, Indexed):
-            A, ii = expressions.ufl_operands
-            if indices == ii:
-                return A
-        return Operator._ufl_expr_reconstruct_(self, expressions, indices)
+    def _simplify_indexed(self, multiindex):
+        """Return a simplified Expr used in the constructor of Indexed(self, multiindex)."""
+        # Untangle as_tensor(C[kk], jj)[ii] -> C[ll]
+        B, jj = self.ufl_operands
+        if isinstance(B, Indexed):
+            C, kk = B.ufl_operands
+            if all(j in kk for j in jj):
+                ii = tuple(multiindex)
+                rep = dict(zip(jj, ii))
+                Cind = tuple(rep.get(k, k) for k in kk)
+                return Indexed(C, MultiIndex(Cind))
+
+        return Operator._simplify_indexed(self, multiindex)
 
     def indices(self):
         """Get indices."""
@@ -181,8 +251,7 @@ class ComponentTensor(Operator):
 
     def evaluate(self, x, mapping, component, index_values):
         """Evaluate."""
-        indices = self.ufl_operands[1]
-        a = self.ufl_operands[0]
+        a, indices = self.ufl_operands
 
         if len(indices) != len(component):
             raise ValueError("Expecting a component matching the indices tuple.")
@@ -200,7 +269,7 @@ class ComponentTensor(Operator):
 
     def __str__(self):
         """Format as a string."""
-        return "{ A | A_{%s} = %s }" % (self.ufl_operands[1], self.ufl_operands[0])
+        return f"{{ A | A_{{{self.ufl_operands[1]}}} = {self.ufl_operands[0]} }}"
 
 
 # --- User-level functions to wrap expressions in the correct way ---
@@ -217,7 +286,7 @@ def numpy2nestedlists(arr):
 
 def _as_list_tensor(expressions):
     """Convert to a list tensor."""
-    if isinstance(expressions, (list, tuple)):
+    if isinstance(expressions, list | tuple):
         expressions = [_as_list_tensor(e) for e in expressions]
         return ListTensor(*expressions)
     else:
@@ -263,11 +332,11 @@ def as_tensor(expressions, indices=None):
             return expressions
 
         # Support numpy array, but avoid importing numpy if not needed
-        if not isinstance(expressions, (list, tuple)):
+        if not isinstance(expressions, list | tuple):
             expressions = from_numpy_to_lists(expressions)
 
         # Sanity check
-        if not isinstance(expressions, (list, tuple, Expr)):
+        if not isinstance(expressions, list | tuple | Expr):
             raise ValueError("Expecting nested list or tuple.")
 
         # Recursive conversion from nested lists to nested ListTensor
@@ -306,13 +375,13 @@ def as_matrix(expressions, indices=None):
             return expressions
 
         # To avoid importing numpy unneeded, it's quite slow...
-        if not isinstance(expressions, (list, tuple)):
+        if not isinstance(expressions, list | tuple):
             expressions = from_numpy_to_lists(expressions)
 
         # Check for expected list structure
-        if not isinstance(expressions, (list, tuple)):
+        if not isinstance(expressions, list | tuple):
             raise ValueError("Expecting nested list or tuple of Exprs.")
-        if not isinstance(expressions[0], (list, tuple)):
+        if not isinstance(expressions[0], list | tuple):
             raise ValueError("Expecting nested list or tuple of Exprs.")
     else:
         if len(indices) != 2:
@@ -331,11 +400,11 @@ def as_vector(expressions, index=None):
             return expressions
 
         # To avoid importing numpy unneeded, it's quite slow...
-        if not isinstance(expressions, (list, tuple)):
+        if not isinstance(expressions, list | tuple):
             expressions = from_numpy_to_lists(expressions)
 
         # Check for expected list structure
-        if not isinstance(expressions, (list, tuple)):
+        if not isinstance(expressions, list | tuple):
             raise ValueError("Expecting nested list or tuple of Exprs.")
     else:
         if not isinstance(index, Index):
